@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8:
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011, 2016  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import absolute_import
 
 import errno
 import locale
@@ -26,7 +25,8 @@ import sqlite3
 import subprocess
 import sys
 import tempfile
-from urlparse import urlparse
+import importlib
+from urllib.parse import urlparse
 import webbrowser
 
 from solfege import mpd
@@ -51,7 +51,7 @@ try:
 except ImportError:
     alsaseq = None
 
-solfege_copyright = u"Copyright © 1999-2008 Tom Cato Amundsen <tca@gnu.org>, and others."
+solfege_copyright = "Copyright © 1999-2008 Tom Cato Amundsen <tca@gnu.org>, and others."
 
 warranty = """
 This program is free software: you can redistribute it and/or modify
@@ -145,9 +145,9 @@ def check_rcfile():
                 " ".join(v[1:]))
     if cfg.get_int("app/rcfileversion") < 17:
         v = cfg.get_string("app/frontpage").split("/")
-        if v[0] == u"exercises" and v[1] != u"standard":
+        if v[0] == "exercises" and v[1] != "standard":
             cfg.set_string("app/frontpage",
-                           u"/".join([v[0], u"standard"] + v[1:]))
+                           "/".join([v[0], "standard"] + v[1:]))
     if cfg.get_int("app/rcfileversion") < 18:
         cfg.del_key("gui/web_browser_as_help_browser")
     if cfg.get_int("app/rcfileversion") < 19:
@@ -162,7 +162,6 @@ def check_rcfile():
             if i > 0:
                 cfg.set_int("%s/inputwidget" % ex, i + 1)
 
-
     cfg.set_int("app/rcfileversion", rcfileversion)
     try:
         a = mpd.notename_to_int(cfg.get_string("user/lowest_pitch"))
@@ -177,6 +176,7 @@ def check_rcfile():
 
 
 class SolfegeApp(cfg.ConfigUtils):
+
     def __init__(self, options):
         """
         options -- command line options parsed by optparse
@@ -196,6 +196,7 @@ class SolfegeApp(cfg.ConfigUtils):
             if os.path.isdir(os.path.join('help', lang)):
                 self.m_userman_language = lang
                 break
+
     def setup_sound(self):
         if sys.platform == 'win32' and \
                     cfg.get_string("sound/type") == "sequencer-device":
@@ -213,7 +214,7 @@ class SolfegeApp(cfg.ConfigUtils):
                 try:
                     soundcard.initialise_alsa_sequencer((clientid, portid),
                             self.m_options.verbose_sound_init)
-                except alsaseq.SequencerError, e:
+                except alsaseq.SequencerError as e:
                     logging.debug("initialise_alsa_sequencer failed. Using fake synth.")
                     self.display_sound_init_error_message(e)
                     soundcard.initialise_using_fake_synth(True)
@@ -231,12 +232,12 @@ class SolfegeApp(cfg.ConfigUtils):
             try:
                 soundcard.initialise_winsynth(cfg.get_int("sound/synth_number"),
                       verbose_init=self.m_options.verbose_sound_init)
-            except ImportError, e:
+            except ImportError as e:
                 self.display_sound_init_error_message(e)
                 cfg.set_string("sound/type", "fake-synth")
                 soundcard.initialise_using_fake_synth(True)
                 return
-            except RuntimeError, e:
+            except RuntimeError as e:
                 # We can get here if winmidi.output_devices() in winsynth
                 # __init__ returns no devices. Don't know when, but it could
                 # happen.
@@ -261,7 +262,7 @@ class SolfegeApp(cfg.ConfigUtils):
                              cfg.get_string("sound/device_file"),
                              cfg.get_int("sound/synth_number"),
                              verbose_init=self.m_options.verbose_sound_init)
-            except (soundcard.SoundInitException, OSError, ImportError), e:
+            except (soundcard.SoundInitException, OSError, ImportError) as e:
                 self.m_sound_init_exception = e
                 soundcard.initialise_using_fake_synth(True)
         if cfg.get_string("programs/csound") == "AUTODETECT":
@@ -280,6 +281,7 @@ class SolfegeApp(cfg.ConfigUtils):
                 break
             else:
                 cfg.set_string("programs/mma", "")
+
     def display_sound_init_error_message(self, e):
         if isinstance(e, soundcard.SoundInitException):
             solfege.win.display_error_message(
@@ -305,6 +307,7 @@ class SolfegeApp(cfg.ConfigUtils):
                 "the preferences window.\n\n"
                 "It is also possible that the OS sound setup is incorrect."
                 % str(e).decode(locale.getpreferredencoding(), 'replace'))
+
     def please_help_me(self):
         if isinstance(solfege.win.get_view(), abstract.Gui):
             # If the view visible is an exercise, when we check if the
@@ -315,9 +318,11 @@ class SolfegeApp(cfg.ConfigUtils):
                 # if not, we display help page named the same as the
                 # exercise module
                 self.handle_href('%s.html' % solfege.win.m_viewer)
+
     def show_exercise_theory(self):
         if self.m_teachers[self.m_running_exercise].m_P.header.theory:
             solfege.win.display_docfile("%s.html" % self.m_teachers[self.m_running_exercise].m_P.header.theory)
+
     def _practise_lessonfile(self, filename, urlobj=None):
         """
         return the module name.
@@ -330,8 +335,8 @@ class SolfegeApp(cfg.ConfigUtils):
             # if the user have written his own exercise module
             if os.path.exists(os.path.normpath(os.path.join(
                             os.path.dirname(filename),
-                            "..", "modules", u"%s.py" % module))):
-                module = u"user:%s/%s" % (
+                            "..", "modules", "%s.py" % module))):
+                module = "user:%s/%s" % (
                     os.path.dirname(filename).split(os.sep)[-2],
                     module)
         if module not in self.m_teachers:
@@ -345,53 +350,64 @@ class SolfegeApp(cfg.ConfigUtils):
         self.m_teachers[module].g_view = solfege.win.box_dict[module]
         solfege.win.show_help_on_current()
         return module
+
     def practise_lessonfile(self, filename):
         def cleanup():
             module = lessonfile.infocache.get(filename, 'module')
-            self.m_teachers[module].m_P = None
-            solfege.win.box_dict[module].practise_box.set_sensitive(False)
-            solfege.win.box_dict[module].config_box.set_sensitive(False)
-            solfege.win.box_dict[module].action_area.set_sensitive(False)
-            solfege.win.box_dict[module].std_buttons_end_practise()
+            if module in self.m_teachers:
+                self.m_teachers[module].m_P = None
+            if module in solfege.win.box_dict:
+                solfege.win.box_dict[module].practise_box.set_sensitive(False)
+                solfege.win.box_dict[module].g_config_grid.set_sensitive(False)
+                solfege.win.box_dict[module].action_area.set_sensitive(False)
+                solfege.win.box_dict[module].std_buttons_end_practise()
         try:
             module = self._practise_lessonfile(filename)
         except (lessonfile.LessonfileParseException,
                 dataparser.DataparserException,
                 parsetree.ParseTreeException,
-                IOError), e:
+                IOError) as e:
             cleanup()
             gu.display_exception_message(e, lessonfile=filename)
             return
+        except abstract.ExerciseModuleImportError as e:
+            solfege.win.display_error_message2(
+                _("Exercise module not found"), str(e))
+            return
         if 'm_discards' in dir(self.m_teachers[module].m_P):
             for msg in self.m_teachers[module].m_P.m_discards:
-                print >> sys.stderr, msg
+                print(msg, file=sys.stderr)
         solfege.win.box_dict[module].practise_box.set_sensitive(True)
-        solfege.win.box_dict[module].config_box.set_sensitive(True)
+        solfege.win.box_dict[module].g_config_grid.set_sensitive(True)
         solfege.win.box_dict[module].action_area.set_sensitive(True)
         solfege.win.box_dict[module].on_start_practise()
         w = solfege.win.g_ui_manager.get_widget("/Menubar/HelpMenu/PerExerciseHelp/HelpTheory")
         if w:
             w.set_sensitive(bool(self.m_teachers[module].m_P.header.theory))
         return module
+
     def test_lessonfile(self, filename):
         self.m_test_mode = True
-        module = self.practise_lessonfile(filename)
+        self.practise_lessonfile(filename)
         solfege.win.enter_test_mode()
+
     def handle_href(self, href):
         u = urlparse(href)
         if u.scheme:
             try:
                 webbrowser.open_new(href)
-            except Exception, e:
+            except Exception as e:
                 solfege.win.display_error_message2(_("Error opening web browser"), str(e))
         else:
             solfege.win.display_docfile(u.path)
+
     def create_teacher(self, modulename):
         """
         Create the teacher in 'modulename' and add it to self.m_teachers.
         """
         m = self.import_module(modulename)
         self.m_teachers[modulename] = m.Teacher(modulename)
+
     def import_module(self, modulename):
         """
         If prefixed with "solfege:"
@@ -414,11 +430,15 @@ class SolfegeApp(cfg.ConfigUtils):
                                       "exercises", collection, "modules")
             sys.path.insert(0, module_dir)
             m = __import__(modulename.split("/")[1])
-            reload(m)
+            importlib.reload(m)
             del sys.path[0]
         else:
-            m = __import__("solfege.exercises.%s" % modulename, fromlist=("solfege.exercises.%s" % modulename,), level=0)
+            try:
+                m = __import__("solfege.exercises.%s" % modulename, fromlist=("solfege.exercises.%s" % modulename,), level=0)
+            except ImportError as e:
+                raise abstract.ExerciseModuleImportError(e)
         return m
+
     def reset_exercise(self, w=None):
         """
         Call on_end_practise, and then on_start_practise in
@@ -427,6 +447,7 @@ class SolfegeApp(cfg.ConfigUtils):
         if isinstance(solfege.win.get_view(), abstract.Gui):
             solfege.win.get_view().on_end_practise()
             solfege.win.get_view().on_start_practise()
+
     def quit_program(self):
         if isinstance(solfege.win.get_view(), abstract.Gui):
             g = solfege.win.get_view()
@@ -438,19 +459,18 @@ class SolfegeApp(cfg.ConfigUtils):
                 g.on_end_practise()
         try:
             cfg.sync()
-        except IOError, e:
+        except IOError as e:
             gu.display_exception_message(e)
         try:
-            solfege.db.conn.commit()
-        except sqlite3.ProgrammingError, e:
-            gu.display_exception_message(e)
-        try:
-            solfege.db.conn.close()
-        except sqlite3.ProgrammingError, e:
+            if solfege.db:
+                solfege.db.conn.commit()
+                solfege.db.conn.close()
+        except sqlite3.ProgrammingError as e:
             gu.display_exception_message(e)
         if soundcard.synth:
             soundcard.synth.close()
         shutil.rmtree(lessonfile.MusicBaseClass.temp_dir, True)
+
     def export_training_set(self, export_data, export_dir, output_format,
                             name_track_by_question):
         """
@@ -462,7 +482,7 @@ class SolfegeApp(cfg.ConfigUtils):
             tempo is a dict of two integers
             """
             track = mpd.Track()
-            track.set_bpm(*tempo)#self.get_int('config/default_bpm'))
+            track.set_bpm(*tempo)  # self.get_int('config/default_bpm'))
             track.note(mpd.Rat(n, 4), 80, 0)
             soundcard.synth.play_track(track)
         track_idx = 0
@@ -528,6 +548,7 @@ class SolfegeApp(cfg.ConfigUtils):
                 else:
                     logging.warning("export_training_set:ignoring exercise with module='%s'", module)
 #####
+
                 def do_convert(from_format, to_format):
                     """
                     Return False if we think the convert failed.
@@ -554,7 +575,7 @@ class SolfegeApp(cfg.ConfigUtils):
                         try:
                             subprocess.call(
                                 [cfg.get_string(app_cfg_name)] + opts)
-                        except OSError, e:
+                        except OSError as e:
                             raise osutils.BinaryForMediaConvertorException(app_cfg_name,
                                 cfg.get_string(app_cfg_name), e)
 
@@ -566,7 +587,7 @@ class SolfegeApp(cfg.ConfigUtils):
                             # because we don't want to display this error for
                             # every single file.
                             output_format = 'midi'
-                            solfege.win.display_error_message2("External program must have failed", "The file in %(from)s format was not generated from the %(to)s file as expected. Please check your setup in the preferences window (CTRL-F12)." % {'to':to_format.upper(), 'from': from_format.upper()})
+                            solfege.win.display_error_message2("External program must have failed", "The file in %(from)s format was not generated from the %(to)s file as expected. Please check your setup in the preferences window (CTRL-F12)." % {'to': to_format.upper(), 'from': from_format.upper()})
                     except (TypeError, KeyError):
                         solfege.win.display_error_message2("%(from)s to %(to)s config error", "There was a format string error. Will not generate WAV files. Please check the app/midi_to_wav_cmd config variable." % {'from': from_format, 'to': to_format})
                         output_format = 'midi'
@@ -583,6 +604,7 @@ class SolfegeApp(cfg.ConfigUtils):
                     del self.m_abort_export
                     return
         reportlib.HtmlReport(report, os.path.join(export_dir, "toc.html"))
+
     def sheet_gen_questions(self, count, sdict):
         """
         count -- how many questions should we generate. We use this value
@@ -601,12 +623,13 @@ class SolfegeApp(cfg.ConfigUtils):
             assert module in ('harmonicinterval', 'melodicinterval')
             for x in self._sheet_gen_question_interval(module, p, count, sdict):
                 yield x
+
     def _sheet_gen_question_idbyname(self, p, count, sdict):
         """
         yield count dicts, where each dict contain the data needed to
         print both the teachers and the students question.
         """
-        counts = {}.fromkeys(range(len(p.m_questions)), 0)
+        counts = {}.fromkeys(list(range(len(p.m_questions))), 0)
         for x in range(count):
             while 1:
                 p.select_random_question()
@@ -628,6 +651,7 @@ class SolfegeApp(cfg.ConfigUtils):
                 ret['answer']['music'] = p.get_lilypond_code()
                 ret['question']['music'] = p.get_lilypond_code_first_note()
                 yield ret
+
     def _sheet_gen_question_interval(self, module, p, count, sdict):
         # FIXME in the idbyname we count how many times each question
         # has been selected, to get an even selection. We don't do it
@@ -675,4 +699,3 @@ class SolfegeApp(cfg.ConfigUtils):
                     teacher.m_tonika.get_octave_notename(),
                     (teacher.m_tonika + teacher.m_interval).get_octave_notename())
                 yield ret
-
